/** \file common.h
  *
  * \brief Defines things which are common to most of the source distribution.
  *
  * If porting to another platform, please check for the presence of stdint.h
  * and if it isn't available, define NO_STDINT_H and check that the typedefs
  * below refer to appropriate types.
  *
  * This file is licensed as described by the file LICENCE.
  */

#ifndef COMMON_H_INCLUDED
#define COMMON_H_INCLUDED

#include <stddef.h>

// Some platforms or toolchains lack stdint.h. Please define NO_STDINT_H and
// modify the three typedefs below if this is the case.
#ifdef NO_STDINT_H

// These typedefs are platform-dependent. Change them if they don't match
// the comments.
/** Unsigned 8 bit integer. */
typedef unsigned char uint8_t;
/** Signed 8 bit integer. */
typedef signed char int8_t;
/** Unsigned 16 bit integer. */
typedef unsigned short uint16_t;
/** Signed 16 bit integer. */
typedef signed short int16_t;
/** Unsigned 32 bit integer. */
typedef unsigned long uint32_t;
/** Signed 32 bit integer. */
typedef signed long int32_t;
/** Unsigned 64 bit integer. */
typedef unsigned __int64 uint64_t;
/** Signed 64 bit integer. */
typedef signed __int64 int64_t;

#else



#ifdef _MSC_VER
/** Unsigned 8 bit integer. */
typedef unsigned char uint8_t;
/** Signed 8 bit integer. */
typedef signed char int8_t;
/** Unsigned 16 bit integer. */
typedef unsigned __int16 uint16_t;
/** Signed 16 bit integer. */
typedef signed __int16 int16_t;
/** Unsigned 32 bit integer. */
typedef unsigned __int32 uint32_t;
/** Signed 32 bit integer. */
typedef signed __int32 int32_t;
/** Unsigned 64 bit integer. */
typedef unsigned __int64 uint64_t;
/** Signed 64 bit integer. */
typedef signed __int64 int64_t;

#else
#include <stdint.h>
#endif

#endif // #ifdef NO_STDINT_H

// Some platforms or toolchains lack stdbool.h. Please define NO_STDBOOL_H
// if this is the case.
#ifdef NO_STDBOOL_H

/** Boolean data type definition for platforms which lack stdbool.h. */
typedef enum BoolEnum
{
	false = 0,
	true = 1
} bool;

#else

#ifdef _MSC_VER
/** Boolean data type definition for platforms which lack stdbool.h. */
typedef enum BoolEnum
{	false = 0,
	true = 1
} bool;

#else

#include <stdbool.h>
#endif

#endif // #ifdef NO_STDBOOL_H



#endif // #ifndef COMMON_H_INCLUDED
