#include "cryptoecc.h"

#ifdef _MSC_VER
#include <string.h>
#endif

#include <string.h>

uECC_Curve uECC_secp256k1(void)
{ return &secp256k1;}

// r, s must be < n
// MIN s,(n-s)
int uECC_iscanonical(uint8_t * s,uECC_Curve curve)
{   int err;
    err = isCanonical(curve,s);
	return err;
}


void uECC_decompress(uint8_t * compressed_point, uint8_t * decompressed_point, uECC_Curve curve)
{   uint8_t key65[65];

    if (ecdsa_uncompress_pubkey(curve,compressed_point,key65) == 0)
	return; //return -1;
	
	memmove(decompressed_point,key65+1,64);
	//return 0;
}



void uECC_compress(uint8_t * decompressed_point, uint8_t * compressed_point, uECC_Curve curve)
{
  curve_point apoint;
  uint8_t key65[65];

  key65[0]=0x04;
  memmove(key65+1,decompressed_point,64);

  if ( ecdsa_read_pubkey(curve,key65,&apoint)== 0)
  return ; //  return -1;

  compress_coords(&apoint,compressed_point);

  return ;

}

int uECC_recover(const uint8_t * decompressed_point, uint8_t * apublic, const uint8_t * hash, unsigned lenhash, const uint8_t * r, const uint8_t * s, uECC_Curve curve)
{ uint8_t sig[64],key65[65] ;
  curve_point apoint;

  if (lenhash != 32)
	  return 0;

  memmove(sig,r,32)        ;
  memmove(sig+32,s,32)     ;
  memset(apublic,0,64)     ;

  key65[0]=0x04;
  memmove(key65+1,decompressed_point,64);

  if ( ecdsa_read_pubkey(curve,key65,&apoint)== 0)
  return 0; 

  memset(key65,0,65);
  if (ecdsa_recover_pub_from_sig2(curve,key65,sig,hash,apoint)==0)  
   {  memmove(apublic,key65+1,64);
     return 1 ;
   }
  return 0;
}

int uECC_make_key(uint8_t * kpublic, uint8_t *kprivate, uECC_Curve curve)
{ static int v=1;
  int i;
  uint8_t key65[65];

  for(i=0;i<32;i++) kprivate[i]=v++;
  
  if (ecdsa_get_public_key65(curve,kprivate,key65) != 0) 
	  return 0; 
  
  memmove(kpublic,key65+1,64);
  return 1;
}

int uECC_verify(const uint8_t * kpublic, const uint8_t * hash, unsigned lenhash, const uint8_t *sig, uECC_Curve curve)
{  uint8_t key65[65] ;

	if (lenhash != 32)
		return 0 ;

	key65[0]=0x04;
	memmove(key65+1,kpublic,64);

	if (ecdsa_verify_digest(curve,key65,sig,hash) != 0)
    return 0;
   
	return 1;
}
